package pop3Server;
//import POP3Protocol;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Hashtable;
import java.util.logging.Logger;

import pop3Server.POP3ClientHandler;
import pop3Server.POP3State;
import pop3Server.AUTHORIZATIONState;

/**
 * Deze klasse verzorgt de complete afhandeling met de cliënt.
 */
public class POP3ClientHandler extends Thread {
	
	private final int CMD_STAT = 1001;
	private final int CMD_LIST = 1002;
	private final int CMD_RETR = 1003;
	private final int CMD_DELE = 1004;
	private final int CMD_RSET = 1005;
	private final int CMD_NOOP = 1006;
	private final int CMD_QUIT = 1007;
	private final int CMD_USER = 1008;
	private final int CMD_PASS = 1009;
	private final int CMD_CAPA = 1010;
	private final int CMD_UIDL = 1011;
	private final int CMD_TOP  = 1012;
	//Commando om DATA Mode te beïndigen.
	//private final int CMD_ENDDATA = 10008;
	private static Hashtable cmdTable;
	
	//Declaratie variabelen
	protected int clientID;
	protected Socket client;
	protected String hostname;
	protected BufferedReader input;
	protected PrintWriter output;
	protected POP3State pop3State;
	protected boolean transferState;

	private Logger log;
	
	public POP3ClientHandler(Socket client, String hostname, int clientID){
		this.client = client;
		this.clientID = clientID;
		this.hostname = hostname;
		
		//Logger
		log = Logger.getLogger("POP3Server");
		
		//Pop3Protocol = new POP3Protocol(hostname);
		//Status goed zetten
		pop3State = new AUTHORIZATIONState();
		
		//Commandotabel bijwerken
		buildCmdTable();
		
		//Writers aanmaken om te kunnen communiceren met de client.
		try{
			if (client != null){
				input = new BufferedReader(new InputStreamReader(client.getInputStream()));
				output = new PrintWriter(new OutputStreamWriter(client.getOutputStream()));
			}
		}
		catch(Exception ex){
			log.warning("client id: " + clientID + " Fout bij initialisatie.");
			ex.printStackTrace();
		}
		
		
	}
	
	/**
	 * Methode om de status te veranderen.
	 * @param smtpState
	 */
	public void setState(POP3State pop3State){
		this.pop3State = pop3State;
	}
	
	
	/**
	 * De methode voor het afwerken van de cliënt
	 */
	public void run(){
		String cmdLine;
		String command;
		Integer cmd;
		int intCmd;
		String cmdArgs = null;
		String bfrOutput = "";

		log.info("client id: " + clientID + " Client thread gestart");
		
		if(input != null && output != null) {
			//Welkoms bericht
			output.println("+OK Welkom op onze POP3 mailserver.");
			//output.println("U bent client: "+clientID);
			output.flush();
			
			try {
				//Controleren of er iets ingevoerd is.
				while ((cmdLine = input.readLine()) != null){					
					//Initialisatie nieuwe commando
					command = cmdLine;
					bfrOutput = null;
					cmdArgs = null;
					
					//Als er een spatie in de input zit, dan commando eruit halen
					int intSpace = cmdLine.indexOf(" ");
					if (intSpace != -1){
						command = cmdLine.substring(0, intSpace);
						cmdArgs = cmdLine.substring(intSpace+1);
					}
					
					//Opzoeken commando in de tabel
					cmd = (Integer)cmdTable.get(command.toUpperCase());
					//Controleren of het commando bestaat.
					if (cmd == null){
						//if (!pop3State.transferState){
							//output.println("500 Ongeldige invoer.");
							bfrOutput = "-ERR Ongeldige invoer.";
						//}
					}
					else{
						intCmd = cmd.intValue();
						switch(intCmd)
						{
							case CMD_STAT:{
								bfrOutput = pop3State.cmdSTAT(this, cmdArgs, hostname);
								log.info("client id: " + clientID + " STAT Command.");
								break;
							}
							case CMD_LIST: {
								bfrOutput = pop3State.cmdLIST(this, cmdArgs);
								log.info("client id: " + clientID + " LIST Command.");
								break;
							}
							case CMD_RETR: {
								bfrOutput = pop3State.cmdRETR(this, cmdArgs);
								log.info("client id: " + clientID + " RETR Command.");
								break;
							} 
							case CMD_DELE: {
								log.info("client id: " + clientID + " DELE Command.");
								bfrOutput = pop3State.cmdDELE(this, cmdArgs);
								break;								
							}
							case CMD_RSET: {
								bfrOutput = pop3State.cmdRSET(this);
								break;																
							} 
							case CMD_NOOP: {
								bfrOutput = pop3State.cmdNOOP();
								log.info("client id: " + clientID + " NOOP Command.");								
								break;																								
							}
							case CMD_QUIT: {
								bfrOutput = pop3State.cmdQUIT(this, cmdArgs);
								log.info("client id: " + clientID + " QUIT Command.");
								break;																								
							}
							case CMD_USER: {
								bfrOutput = pop3State.cmdUSER(this, cmdArgs);
								log.info("client id: " + clientID + " USER Command.");
								break;																								
							}
							case CMD_PASS: {
								bfrOutput = pop3State.cmdPASS(this, cmdArgs);
								log.info("client id: " + clientID + " PASS Command.");
								break;																								
							}
							case CMD_CAPA: {
								//bfrOutput = pop3State.cmdCapa(this, cmdArgs);
								bfrOutput = "+OK Lijst volgt." + System.getProperty("line.separator") + "PLAIN\r" + System.getProperty("line.separator") + "USER\r" + System.getProperty("line.separator") + "UIDL\r" + System.getProperty("line.separator") + ".\r"; //Tijdelijk
								log.info("client id: " + clientID + " CAPA Command.");
								break;
							}
							case CMD_UIDL: {
								bfrOutput = pop3State.cmdUIDL(this, cmdArgs);
								log.info("client id: " + clientID + " UIDL Command.");
								break;
							}
							case CMD_TOP: {
								bfrOutput = pop3State.cmdTOP(this, cmdArgs);
								log.info("client id: " + clientID + " TOP Command.");
								break;
							}
							
						}
						
					}
					if (bfrOutput != null){
						if (bfrOutput.equalsIgnoreCase("QUIT")){
							input.close();
							output.close();
							client.close();
							log.info("client id: " + clientID + " verbinding verbroken.");
							return;							
						} 
						//Elseif QUITTRANS
						else{
							output.println(bfrOutput);	
						}
						output.flush();
					}
				}
			}
			catch(Exception ex){
				log.warning("client id: " + clientID + " Fout in commando afhangeling opgetreden.");
				ex.printStackTrace();
			}

			
		}
	}	
	
	/**
	 * Deze methode bouwt een hashtable op met alle commands.
	 * Deze wordt gebruikt om ingevoerde commando's snel te detecteren.
	 */
	private void buildCmdTable(){
		cmdTable = new Hashtable();
		cmdTable.put("STAT", new Integer(CMD_STAT));	
		cmdTable.put("LIST", new Integer(CMD_LIST));
		cmdTable.put("RETR", new Integer(CMD_RETR));		
		cmdTable.put("DELE", new Integer(CMD_DELE));
		cmdTable.put("RSET", new Integer(CMD_RSET));	
		cmdTable.put("NOOP", new Integer(CMD_NOOP));
		cmdTable.put("QUIT", new Integer(CMD_QUIT));
		cmdTable.put("USER", new Integer(CMD_USER));
		cmdTable.put("PASS", new Integer(CMD_PASS));
		cmdTable.put("CAPA", new Integer(CMD_CAPA));
		cmdTable.put("UIDL", new Integer(CMD_UIDL));
		cmdTable.put("TOP", new Integer(CMD_TOP));
		//cmdTable.put(".", new Integer(CMD_ENDDATA));
	}
	
}
