package smtpServer;

import java.util.logging.Logger;
import java.util.regex.*;

import data.*;

/**
 * Abstracte klasse om de status van de SMTPClientHandler bij te houden.
 * 
 * @version 0.50
 * TODO Zorgen dat er meerdere ontvangers meegegeven mogen worden.
 * TODO Commando NOOP implementeren
 * TODO Verbeteren controle op ingevoerde gegevens (bijv. sender < 64)
 * TODO [OPT] Regular Expression verbeteren voor controle mailadres.
 * TODO [OPT] Implementeren extra foutcodes SMTP.
 * TODO [OPT] Implementeren van RFC1123, RFC1869 (deze voegen extra func. toe.).
 */

public abstract class SMTPState {
	//Declaratie variabelen
	//Lokale variabelen
	protected String REPLY_500 = "500 Ongeldige invoer";
	protected boolean dataState;
	
	//Message variabelen
	protected static Message msg;
	protected static String msgData;
		
	//Abstracte methodes (State-Pattern)
	public abstract String cmdHELO(SMTPClientHandler smtpCH, String cmdArgs, String host);
	public abstract String cmdMAIL(SMTPClientHandler smtpCH, String cmdArgs);
	public abstract String cmdRCPT(SMTPClientHandler smtpCH, String cmdArgs);
	public abstract String cmdDATA(SMTPClientHandler smtpCH, String cmdArgs);
	public abstract String cmdEndData(SMTPClientHandler smtpCH, String cmdArgs);	
	
	/**
	 * Voert het RSET commando uit.
	 * @param smtpCH
	 * @return 250 OK
	 */
	public String cmdRSET(SMTPClientHandler smtpCH){
		smtpCH.setState(new WAITState());
		return "250 OK Reset";
	};
	
	/**
	 * Voert het NOOP commando uit.
	 * @return
	 */
	public String cmdNOOP(){
		return null;
	};
	
	/**
	 * Voert het QUIT commando uit.
	 * @return QUIT
	 */
	public String cmdQUIT(){
		return "QUIT";
	};
	
	/**
	 * Zorgt ervoor dat er gescwitched kan worden tussen datastate.
	 * Dit zorgt ervoor dat ook commando's in de mailbody kunnen worden gezet.
	 * @param state
	 */
	public void setDataState(boolean state){
		dataState = state;
	}
	
	/**
	 * Geeft terug of de connectie zich in DATA status bevindt.
	 * @return
	 */
	public boolean getDataState(){
		return dataState;
	}
	
	/**
	 * Zorgt ervoor dat het bericht gemaakt wordt.
	 * In datastatus, moet alle invoer in het bericht komn.
	 * @param input
	 */
	public void setMsgData(String input){
		msgData = msgData + input + "\r\n";
	}
	
	/**
	 * Controlleerd of een mailadres wel voldoet aan de eisen.
	 * In de controle worden "<" en ">" weggelaten.
	 * @param mail
	 * @return
	 */
	protected boolean isValidMail(String mail){
		//Dirty maar werkt :-)
		mail = mail.replaceAll("<", "");
		mail = mail.replaceAll(">", "");
		Pattern mailPattern = Pattern.compile(".+@.+\\.[a-z]+");
		Matcher mailMatcher = mailPattern.matcher(mail);
		if (mailMatcher.matches()){
			return true;
		}
		else {
			return false;
		}
	}
	
	/**
	 * Het verwerken van het MAIL Commando
	 * Deze kan nml. in HELO en in WAIT State worden aangeroepen
	 * @param cmdArgs
	 * @param smtpCH
	 * @return 501 of 250
	 */
	//Had ook gekund door geen abstracte methode aan te maken en dan middels overerving.
	protected String doMail(SMTPClientHandler smtpCH, String cmdArgs){
		String parser = "FROM:";
		//Controle toevoegen of er wel echt een mailadres is meegegeven
		if (cmdArgs == null){
			return "501 Ongeldige invoer. (vb. 'MAIL FROM: ik@jouwdomein.nl')";
		}		 
		//Controleren of de lengte van het argument langer is als "FROM:"
		else if (cmdArgs.length() <= parser.length()){
			return "501 Ongeldige invoer. (vb. 'MAIL FROM: ik@jouwdomein.nl')";
		}
		//Controleren of er ook echt FROM: is meegegeven.
		else if (!cmdArgs.substring(0, parser.length()).toUpperCase().equals(parser)) {
			return "501 Ongeldige invoer. (vb. 'MAIL FROM: ik@jouwdomein.nl')";
		}
		else if (!isValidMail(cmdArgs)){
			return "501 Ongeldige invoer. (vb. 'MAIL FROM: ik@jouwdomein.nl')";
		}
		else{
			//De FROM: wegfilteren
			cmdArgs = cmdArgs.substring(parser.length());
			msg.setFrom(cmdArgs);
			smtpCH.setState(new MAILState());
			return "250 OK Mail van: "+cmdArgs;
		}
	}
}
